import React, { useRef, useState } from 'react';

import {
  AtomicBlockUtils,
  ContentBlock,
  DraftHandleValue,
  EditorState,
  getDefaultKeyBinding,
  RichUtils,
} from 'draft-js';

import './editor.less';
import { Editor as DraftEditor, convertToRaw } from 'draft-js';
import './Draft.less';
import { stateToHTML } from 'draft-js-export-html';
import Toolbar from '../components/toolbar/Toolbar';
import * as blockUtils from '../utils/blockUtils';
// import { DraftHandleValue } from '_@types_draft-js@0.11.9@@types/draft-js';
import ResizeableImage from '../components/resizeable-image/ResizeableImage';

interface IProps {
  placeholder?: string;
  readonly?: boolean;
  /**
   * track state change
   * @param state string  editor state
   * @param content html content
   */
  onChange?: (state: string, content: string) => void;
  uploadImage?: (file: Blob) => {
    url: string;
    width?: number;
    height?: number;
  };
}

const Editor = (props: IProps): JSX.Element => {
  const {
    placeholder = 'Please input something...',
    onChange,
    uploadImage,
    readonly = false,
  } = props;
  const [editorState, setEditorState] = useState<EditorState>(() =>
    EditorState.createEmpty(),
  );

  const editorRef = useRef<DraftEditor>(null);
  const onStateChange = state => {
    if (
      readonly ||
      (blockUtils.getSelectedBlocksType(state) === 'atomic' &&
        state.getSelection().isCollapsed)
    ) {
      return;
    }
    setEditorState(state);
    setTimeout(() => {
      if (onChange) {
        onChange(
          JSON.stringify(convertToRaw(state.getCurrentContent())),
          stateToHTML(state.getCurrentContent()),
        );
      }
    });
  };

  const handleKeyCommand = (command, editorState) => {
    const newState = RichUtils.handleKeyCommand(editorState, command);
    if (newState) {
      setEditorState(newState);
      return 'handled';
    }
    return 'not-handled';
  };
  const focusEditor = () => {
    if (editorRef.current) editorRef.current.focus();
  };
  const mapKeyToEditorCommand = event => {
    if (event.keyCode === 9) {
      const newEditorState = RichUtils.onTab(event, editorState, 4);
      if (newEditorState !== editorState) {
        setEditorState(newEditorState);
      }
      return null;
    }
    return getDefaultKeyBinding(event);
  };

  const handlePasteImage = (files: Blob[]): DraftHandleValue => {
    const file = files[0];
    const type = file ? file.type : '';
    const acceptFileTypes = [
      'image/png',
      'image/jpg',
      'image/jpeg',
      'image/gif',
    ];
    if (acceptFileTypes.indexOf(type) === -1) {
      alert('only image accepted');
      return 'handled';
    }
    if (uploadImage) {
      const result = uploadImage(file);
      const newState = insertImage(result.url, result.width, result.height);
      onStateChange(newState);
    }

    return 'handled';
  };

  const insertImage = (url?: string, width?: number, height?: number) => {
    const contentState = editorState.getCurrentContent();
    const contentStateWithEntity = contentState.createEntity(
      'IMAGE',
      'MUTABLE',
      { src: url, width, height },
    );
    const entityKey = contentStateWithEntity.getLastCreatedEntityKey();
    return AtomicBlockUtils.insertAtomicBlock(editorState, entityKey, ' ');
  };

  const blockRender = (contentBlock: ContentBlock) => {
    const type = contentBlock.getType();

    if (type === 'atomic') {
      const entity = editorState
        .getCurrentContent()
        .getEntity(contentBlock.getEntityAt(0));
      const entityData = entity.getData();
      const entityType = entity.getType();
      switch (entityType) {
        case 'IMAGE':
          return {
            component: ResizeableImage,
            editable: false,
            props: {
              src: entityData.src,
              width: entityData.width,
              height: entityData.height,
            },
          };
      }
    }
    return;
  };

  return (
    <div className={'RichEditor-root'} style={{ minHeight: '300px' }}>
      <Toolbar editorState={editorState} onChange={onStateChange} />
      <div className={'RichEditor-editor'} onClick={focusEditor}>
        <DraftEditor
          ref={editorRef}
          placeholder={placeholder}
          editorState={editorState}
          handleKeyCommand={handleKeyCommand}
          keyBindingFn={mapKeyToEditorCommand}
          onChange={onStateChange}
          handlePastedFiles={handlePasteImage}
          blockRendererFn={blockRender}
          spellCheck={true}
        />
      </div>
    </div>
  );
};

export default Editor;
